import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public class FigurFrame extends JFrame implements ActionListener {

    private JButton erzeugenButton;
    private JButton loeschenButton;
    private JButton bewegenButton;

    private JTextArea ausgabeBereich;

    private Figur aktuelleFigur;

    /**
     * In diesem Programmstueck wird das Fenster erzeugt. Auf dem Fenster sind
     * spaeter drei Buttons fuer Erzeugen, Loeschen und Bewegen zu sehen. Ausserdem
     * gibt es einen Ausgabebereich, in dem ausgegeben wird, welche Aktion das
     * Programm gerade ausgefuehrt hat.
     */
    public FigurFrame() {
        /* In der Kopfleiste des Fenster steht "FigurFrame" */
        this.setTitle("FigurFrame");

        /* Das Fenster ist 700 Pixel breit und 200 Pixel hoch. */
        this.setSize(700, 200);

        /* Hier werden alle Buttons erzeugt. */
        this.erzeugenButton = new JButton("Erzeugen");
        this.loeschenButton = new JButton("Loeschen");
        this.bewegenButton = new JButton("Bewegen");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(10, 100);

        /* Hier werden alle Buttons zusammengruppiert. */
        JPanel buttonPanel = new JPanel();
        buttonPanel.add(this.erzeugenButton);
        buttonPanel.add(this.loeschenButton);
        buttonPanel.add(this.bewegenButton);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden koennen.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(2, 1));
        /* Hier wird die Gruppe von Buttons platziert. */
        contentPane.add(buttonPanel);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(this.ausgabeBereich);

        /*
         * Hier wird der FigurFrame als Listener fuer Knopfdruck-Ereignisse bei
         * jedem der drei Buttons registriert.
         */
        this.erzeugenButton.addActionListener(this);
        this.loeschenButton.addActionListener(this);
        this.bewegenButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrueckt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemaess beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    /**
     * Dieses Programmstueck wird immer dann ausgeführt, wenn ein Benutzer auf
     * einen Button drueckt.
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == this.erzeugenButton) {
            this.erzeugen();
        }
        else if (source == this.loeschenButton) {
            this.loeschen();
        }
        else if (source == this.bewegenButton) {
            this.bewegen();
        }
    }

    /**
     * Diese Methode erzeugt eine neue Figur, falls es momentan keine Figur
     * gibt.
     */
    private void erzeugen() {
        if (this.aktuelleFigur != null) {
            this.ausgabeBereich
                    .setText("Es existiert bereits eine Figur. "
                            + "Sie muessen diese Figur zuerst loeschen, "
                            + "bevor eine neue Figur erzeugt werden kann!");
        }
        else {
            String einlesenPositionX = JOptionPane
                    .showInputDialog("Mittelpunkt x-Position: ");
            int xpos = Integer.parseInt(einlesenPositionX);

            String einlesenPositionY = JOptionPane
                    .showInputDialog("Mittelpunkt y-Position: ");
            int ypos = Integer.parseInt(einlesenPositionY);

            String farbe = JOptionPane.showInputDialog("Farbe: ");

            String einlesenFuellung = JOptionPane
                    .showInputDialog("Fuellung [true/false]: ");
            boolean fuellung = Boolean.parseBoolean(einlesenFuellung);

            this.aktuelleFigur = new Figur(xpos, ypos, farbe,
                    fuellung);

            this.ausgabeBereich
                    .setText("Folgendes Objekt wurde erzeugt: "
                            + this.aktuelleFigur
                            + " mit dem Mittelpunkt: "
                            + this.aktuelleFigur.getMittelpunkt()
                                    .getX()
                            + ", "
                            + this.aktuelleFigur.getMittelpunkt()
                                    .getY());
        }
    }

    /**
     * Diese Methode loescht die aktuelle Figur, falls es eine gibt.
     */
    private void loeschen() {
        if (this.aktuelleFigur == null) {
            this.ausgabeBereich.setText("Es existiert keine Figur, "
                    + "die geloescht werden kann.");
        }
        else {
            this.aktuelleFigur = null;
            this.ausgabeBereich.setText("Die Figur wurde geloescht.");
        }
    }

    /**
     * Diese Methode bewegt die aktuelle Figur, falls es eine gibt.
     */
    private void bewegen() {
        if (this.aktuelleFigur == null) {
            this.ausgabeBereich.setText("Es existiert keine Figur, "
                    + "die bewegt werden kann.");

        }
        else {
            String einlesenDx = JOptionPane
                    .showInputDialog("Objekt bewegen um x-Wert: ");
            int dx = Integer.parseInt(einlesenDx);

            String einlesenDy = JOptionPane
                    .showInputDialog("Objekt bewegen um y-Wert: ");
            int dy = Integer.parseInt(einlesenDy);

            this.aktuelleFigur.bewegen(dx, dy);

            this.ausgabeBereich.setText("Das Objekt "
                    + this.aktuelleFigur
                    + " hat nun den Mittelpunkt: "
                    + this.aktuelleFigur.getMittelpunkt().getX()
                    + ", "
                    + this.aktuelleFigur.getMittelpunkt().getY());
        }
    }
}
